import re


def get_alias_map():

    d = {"ptf": "mcp", "smf": "smp"}

    return d


def alias_to_path(alias: str, custom_aliases: dict = {}):
    """
    Add default aliases here to map them to the correct path
    """
    d = get_alias_map()
    d = {**d, **custom_aliases}

    return d.get(alias, alias)


def get_path_template(template: str, label: str):

    template_map_d = {
        "ng-sgp": {
            "prefix": "data",
            "prev": "sgp",
            "root": "natural-gas",
        },
        "ng-vgp": {
            "prefix": "data",
            "prev": "vgp",
            "root": "natural-gas",
        },
        "ng-tr": {
            "prefix": "data",
            "prev": "transmission",
            "root": "natural-gas",
        },
        "ng-general": {
            "prefix": "data",
            "prev": "general-data",
            "root": "natural-gas",
        },
        "ra": {
            "prefix": "data",
            "prev": "retroactive-adjustment",
        },
        "yek-g": {
            "prefix": "data",
            "prev": "yek-g",
        },
        "vep": {
            "prefix": "data",
            "prev": "pfm",
        },
        "cons": {
            "prefix": "data",
            "prev": "consumption",
        },
    }

    d = template_map_d[template]
    d["label"] = label

    return d


def get_path_map(just_call_keys: bool = False):
    path_map = {
        "call": {
            ## GÖP İşlem Hacmi
            "dam-volume": {
                "prefix": "data",
                "prev": "dam",
                "label": "day-ahead-market-trade-volume",  ## GÖP İşlem Hacmi
            },
            ## Fiyattan bağımsız satış teklifi
            "pi-offer": {
                "prefix": "data",
                "prev": "dam",
                "label": "price-independent-offer",  ## Fiyattan bağımsız satış teklifi
            },
            ## Fiyattan bağımsız alış teklifi
            "pi-bid": {
                "prefix": "data",
                "prev": "dam",
                "label": "price-independent-bid",
            },
            ## Arz talep
            "supply-demand": {
                "prefix": "data",
                "prev": "dam",
            },
            ## GÖP Teklif Edilen Alış Miktarları
            "dam-bid": {
                "prefix": "data",
                "prev": "dam",
                "label": "submitted-bid-order-volume",
            },
            ## GÖP Teklif Edilen Satış Miktarları
            "dam-offer": {
                "prefix": "data",
                "prev": "dam",
                "label": "submitted-sales-order-volume",
            },
            ## GÖP Blok Alış Miktarı
            "dam-block-bid": {
                "prefix": "data",
                "prev": "dam",
                "label": "amount-of-block-buying",
            },
            ## GÖP Blok Satış Miktarı
            "dam-block-offer": {
                "prefix": "data",
                "prev": "dam",
                "label": "amount-of-block-selling",
            },
            ## GÖP Esnek Alış Teklif Miktarı Listeleme Servisi
            "dam-flexible-bid": {
                "prefix": "data",
                "prev": "dam",
                "label": "flexible-offer-buying-quantity",
            },
            ## GÖP Esnek Satış Miktarı
            "dam-flexible-offer": {
                "prefix": "data",
                "prev": "dam",
                "label": "flexible-offer-selling-quantity",
            },
            ## GÖP Esnek Teklif Eşleşme Miktarları
            "dam-flexible-matching": {
                "prefix": "data",
                "prev": "dam",
                "label": "matched-flexible-offer-quantity",
            },
            ## GÖP Eşleşme Miktarı
            "dam-clearing": {
                "prefix": "data",
                "prev": "dam",
                "label": "clearing-quantity",
            },
            ## Göp Eşleşme Miktarı Organizasyon Listeleme
            "dam-clearing-org-list": {
                "prefix": "data",
                "prev": "dam",
                "label": "clearing-quantity-organization-list",
            },
            ## GÖP Fark Tutarı
            "dam-diff": {
                "prefix": "data",
                "prev": "dam",
                "label": "side-payments",
            },
            ## GİP Ağırlıklı Ortalama Fiyat
            "wap": {
                "prefix": "data",
                "prev": "idm",
                "label": "weighted-average-price",
            },
            ## GİP Eşleşme Miktarı
            "idm-qty": {
                "prefix": "data",
                "prev": "idm",
                "label": "matching-quantity",
            },
            ## GİP Min - Maks Alış Teklif Fiyatı
            "idm-mm-bid": {
                "prefix": "data",
                "prev": "idm",
                "label": "min-max-bid-price",
            },
            ## GİP Min - Maks Satış Teklif Fiyatı
            "idm-mm-offer": {
                "prefix": "data",
                "prev": "idm",
                "label": "min-max-sales-offer-price",
            },
            ## GİP Min - Maks Eşleşme Fiyatı
            "idm-mm-matching": {
                "prefix": "data",
                "prev": "idm",
                "label": "min-max-matching-price",
            },
            ## GİP İşlem Hacmi
            "idm-volume": {
                "prefix": "data",
                "prev": "idm",
                "label": "trade-value",
            },
            ## GİP İşlem Akışı
            "idm-log": {
                "prefix": "data",
                "prev": "idm",
                "label": "transaction-history",
            },
            ## GİP Teklif Edilen Alış Satış Miktarları
            "idm-ob-qty": {
                "prefix": "data",
                "prev": "idm",
                "label": "bid-offer-quantities",
            },
            ## SMF
            "smp": {"prefix": "data", "prev": "bpm", "label": "system-marginal-price"},
            ## SMF Yön
            "smp-dir": {"prefix": "data", "prev": "bpm", "label": "system-direction"},
            ## YAL Talimat Miktarı
            "bpm-up": {"prefix": "data", "prev": "bpm", "label": "order-summary-up"},
            ## YAT Talimat Miktarı
            "bpm-down": {
                "prefix": "data",
                "prev": "bpm",
                "label": "order-summary-down",
            },
            ## İkili Anlaşma (İA) Alış Miktarı
            "bi-long": {
                "prefix": "data",
                "prev": "bilateral-contracts",
                "label": "bilateral-contracts-bid-quantity",
            },
            ## İkili Anlaşma (İA) Satış Miktarı
            "bi-short": {
                "prefix": "data",
                "prev": "bilateral-contracts",
                "label": "bilateral-contracts-offer-quantity",
            },
            ### EÜAŞ - GTŞ İkili Anlaşmalar
            "bi-euas": {
                "prefix": "data",
                "prev": "bilateral-contracts",
                "label": "amount-of-bilateral-contracts",
            },
            ## Dengesizlik Miktarı
            "imb-qty": {
                "prefix": "data",
                "prev": "imbalance",
                "label": "imbalance-quantity",
            },
            ## Dengesizlik Tutarı
            "imb-vol": {
                "prefix": "data",
                "prev": "imbalance",
                "label": "imbalance-amount",
            },
            ## Dengeden Sorumlu Grup (DSG) Dengesizlik Miktarı
            "imb-qty-g": {
                "prefix": "data",
                "prev": "imbalance",
                "label": "dsg-imbalance-quantity",
            },
            ## DSG Organizasyon Listesi
            "imb-org-list": {
                "prefix": "data",
                "prev": "imbalance",
                "label": "dsg-organization-list",
            },
            ## PTF
            "mcp": {"prefix": "data", "prev": "dam"},
            ## Kesinleşmemiş PTF
            "interim-mcp": {"prefix": "data", "prev": "dam"},
            ## Kesinleşmemiş PTF yayınlandı mı?
            "interim-mcp-status": {
                "prefix": "data",
                "prev": "dam",
                "label": "interim-mcp-published-status",
            },
            ### PTF-SMF-SDF
            "mcp-smp-imb": {
                "prefix": "data",
                "prev": "reporting-service",
                "label": "ptf-smf-sdf",
            },
            ## DGP Talimat Ağırlıklı Ortalama
            "bpm-orders-w-avg": {
                "prefix": "data",
                "prev": "reporting-service",
                "label": "dgp-talimat-agr-ort",
            },
            ## DGP Talimat Ağırlıklı Ortalama
            "bpm-orders": {
                "prefix": "data",
                "prev": "reporting-service",
                "label": "dgp-talimat",
            },
            ## KGÜP
            "kgup": {
                "prefix": "data",
                "prev": "generation",
                "label": "dpp",
            },
            ## KGÜP - İlk Versiyon
            "kgup-v1": {
                "prefix": "data",
                "prev": "generation",
                "label": "dpp-first-version",
            },
            ## KUDUP
            "kudup": {
                "prefix": "data",
                "prev": "generation",
                "label": "sbfgp",
            },
            ## EAK
            "eak": {
                "prefix": "data",
                "prev": "generation",
                "label": "aic",
            },
            ## Gerçek Zamanlı Üretim
            "rt-gen": {
                "prefix": "data",
                "prev": "generation",
                "label": "realtime-generation",
            },
            ## UEVM
            "uevm": {
                "prefix": "data",
                "prev": "generation",
                "label": "injection-quantity",
            },
            ## Santral Listeleme
            "pp-list": {
                "prefix": "data",
                "prev": "generation",
                "label": "powerplant-list",
            },
            ## Santral Listeleme
            "uevm-pp-list": {
                "prefix": "data",
                "prev": "generation",
                "label": "injection-quantity-powerplant-list",
            },
            ## Üretici Organizasyon Listesi
            "gen-org": {
                "prefix": "data",
                "prev": "generation",
                "label": "organization-list",
            },
            ## Bölge Listesi
            "region-list": {
                "prefix": "data",
                "prev": "generation",
                "label": "region-list",
            },
            ## PMS Mesaj Tipi Listesi
            "mms-message-type-list": {
                "prefix": "data",
                "prev": "markets",
                "label": "umm-message-type-list",
            },
            ## PMS Bölge Listeleme Servisi
            "mms-region-list": {
                "prefix": "data",
                "prev": "markets",
                "label": "umm-region-list",
            },
            ## Üretici UEVÇB Listesi
            "gen-uevcb": {
                "prefix": "data",
                "prev": "generation",
                "label": "uevcb-list",
            },
            ## Lisanslı Santral Yatırımları
            "lic-pp-list": {
                "prefix": "data",
                "prev": "generation",
                "label": "licensed-powerplant-investment-list",
            },
            ## Talep Tahmini Listeleme Servisi
            "long-term-demand-forecast": {
                "prefix": "data",
                "prev": "consumption",
                "label": "demand-forecast",
            },
            ## Tüketici Sayısı Servisi
            "consumer-breakdown": {
                "prefix": "data",
                "prev": "consumption",
                "label": "consumer-quantity",
            },
            ## Tüketim Miktarları (İl ve Profil Bazında)
            "consumption-breakdown": {
                "prefix": "data",
                "prev": "consumption",
                "label": "consumption-quantity",
            },
            ## Dağıtım Bölgesi Servisi
            "distribution-region-list": {
                "prefix": "data",
                "prev": "consumption",
                "label": "distribution-region",
            },
            ## Yük Tahmin Planı
            "load-plan": {
                "prefix": "data",
                "prev": "consumption",
                "label": "load-estimation-plan",
            },
            ## Gerçek Zamanlı Tüketim
            "rt-cons": {
                "prefix": "data",
                "prev": "consumption",
                "label": "realtime-consumption",
            },
            ## UEÇM
            "uecm": {
                "prefix": "data",
                "prev": "consumption",
                "label": "uecm",
            },
            ## Serbest Tüketici UEÇM
            "st-uecm": {
                "prefix": "data",
                "prev": "consumption",
                "label": "st-uecm",
            },
            ## Tedarik Yükümlülüğü Kapsamındaki UEÇM
            "su-uecm": {
                "prefix": "data",
                "prev": "consumption",
                "label": "withdrawal-quantity-under-supply-liability",
            },
            ## Yan Hizmetler - Primer Frekans Rezerv Miktarı
            "anc-pf-qty": {
                "prefix": "data",
                "prev": "ancillary-services",
                "label": "primary-frequency-capacity-amount",
            },
            ## Yan Hizmetler - Primer Frekans Kapasite fiyatı (PFK)
            "anc-pfk": {
                "prefix": "data",
                "prev": "ancillary-services",
                "label": "primary-frequency-capacity-price",
            },
            ## Yan Hizmetler - Sekonder Frekans Rezerv Miktarı
            "anc-sf-qty": {
                "prefix": "data",
                "prev": "ancillary-services",
                "label": "secondary-frequency-capacity-amount",
            },
            ## Yan Hizmetler - Sekonder Frekans Kapasite fiyatı (SFK)
            "anc-sfk": {
                "prefix": "data",
                "prev": "ancillary-services",
                "label": "secondary-frequency-capacity-price",
            },
            ## YEKDEM RES Üretim ve Tahmin Listeleme
            "wind-forecast": {
                "prefix": "data",
                "prev": "renewables",
                "label": "res-generation-and-forecast",
            },
            ## YEKDEM Santral Listesi
            "ren-pp-list": {
                "prefix": "data",
                "prev": "renewables",
                "label": "licensed-powerplant-list",
            },
            ## YEKDEM Gerçek Zamanlı Üretim
            "ren-rt-gen": {
                "prefix": "data",
                "prev": "renewables",
                "label": "licensed-realtime-generation",
            },
            ## YEKDEM UEVM
            "ren-uevm": {
                "prefix": "data",
                "prev": "renewables",
                "label": "renewable-sm-licensed-injection-quantity",
            },
            ## Lisanssız Üretim
            "ren-ul-gen": {
                "prefix": "data",
                "prev": "renewables",
                "label": "unlicensed-generation-amount",
            },
            ## Lisanssız Üretim Bedeli
            "ren-ul-cost": {
                "prefix": "data",
                "prev": "renewables",
                "label": "unlicensed-generation-cost",
            },
            ## YEK Bedeli
            # "yekbed": {"redirect": "ren-lic-cost"},
            "ren-lic-cost": {
                "prefix": "data",
                "prev": "renewables",
                "label": "licensed-generation-cost",
            },
            ## YEK Geliri
            "ren-income": {
                "prefix": "data",
                "prev": "renewables",
                "label": "renewables-support-mechanism-income",
            },
            ## YEK Toplam Gider (YEKTOB)
            "ren-total-cost": {
                "prefix": "data",
                "prev": "renewables",
                "label": "total-cost",
            },
            ## Lisanssız Üretim Bedeli
            "ren-capacity": {
                "prefix": "data",
                "prev": "renewables",
                "label": "new-installed-capacity",
            },
            ## YEKDEM Birim Maliyeti
            "ren-unit-cost": {
                "prefix": "data",
                "prev": "renewables",
                "label": "unit-cost",
            },
            ## YEKDEM Katılımcı Listesi
            "ren-participant-list": {
                "prefix": "data",
                "prev": "renewables",
                "label": "renewables-participant",
            },
            ## Sıfır Bakiye Düzeltme Tutarı
            "zero-balance": {
                "prefix": "data",
                "prev": "transmission",
                "label": "zero-balance",
            },
            ## İSKK
            "iskk": {
                "prefix": "data",
                "prev": "transmission",
                "label": "iskk-list",
            },
            ## Kısıt Maliyeti
            "congestion-cost": {
                "prefix": "data",
                "prev": "transmission",
                "label": "congestion-cost",
            },
            ##ENTSO-E (X) Kodları
            "eic-x-org-list": {
                "prefix": "data",
                "prev": "transmission",
                "label": "organization-list",
            },
            ##ENTSO-E (W) Kodları
            "eic-w-org-list": {
                "prefix": "data",
                "prev": "transmission",
                "label": "entso-w-organization",
            },
            ##ENTSO-E (W) Kodları UEVCB
            "eic-w-uevcb-list": {
                "prefix": "data",
                "prev": "transmission",
                "label": "entso-w-uevcb",
            },
            ## Enterkonneksiyon Arıza Bakım Bildirimleri
            "international-line-events": {
                "prefix": "data",
                "prev": "transmission",
                "label": "international-line-events",
            },
            ##Enterkonneksiyon Kapasitesine İlişkin Yıl Öncesi Tahminler
            "tcat-pre-year-forecast": {
                "prefix": "data",
                "prev": "transmission",
                "label": "tcat-pre-year-forecast",
            },
            ##Enterkonneksiyon Kapasitesine İlişkin Ay Öncesi Tahminler
            "tcat-pre-month-forecast": {
                "prefix": "data",
                "prev": "transmission",
                "label": "tcat-pre-month-forecast",
            },
            ##Enterkonneksiyon Hat Kapasiteleri
            "line-capacities": {
                "prefix": "data",
                "prev": "transmission",
                "label": "line-capacities",
            },
            ##Enterkonneksiyon Hat Kapasiteleri
            "capacity-demand": {
                "prefix": "data",
                "prev": "transmission",
                "label": "line-capacities",
            },
            ##Nomine Kapasite
            "nominal-capacity": {
                "prefix": "data",
                "prev": "transmission",
                "label": "nominal-capacity",
            },
            ##Enterkonneksiyon Hat Kapasiteleri Yön Listesi
            "intl-direction-list": {
                "prefix": "data",
                "prev": "transmission",
                "label": "line-capacities-direction",
            },
            ##Enterkonneksiyon Hat Kapasite Talepleri Yön Listesi
            "intl-capacity-demand-direction-list": {
                "prefix": "data",
                "prev": "transmission",
                "label": "line-capacities-direction",
            },
            ## Piyasa Mesaj Sistemi
            "mms": {
                "prefix": "data",
                "prev": "markets",
                "label": "market-message-system",
            },
            ## Organizasyona Göre Santral Listeleme (Piyasa Mesaj Sistemi)
            "mms-pp-list": {
                "prefix": "data",
                "prev": "markets",
                "label": "power-plant-list-by-organization-id",
            },
            ## UEVÇB'ye Göre Santral Listeleme (Piyasa Mesaj Sistemi)
            "mms-uevcb-list": {
                "prefix": "data",
                "prev": "markets",
                "label": "uevcb-list-by-power-plant-id",
            },
            ## İlçe Listeleme
            "district-list": {
                "prefix": "main",
                "prev": "electricity-service",
                "label": "district-list",
            },
            ## Profil Grubu Listeleme
            "profile-group-list": {
                "prefix": "data",
                "prev": "consumption",
                "label": "consumer-sector-list",
            },
            ## Şehir Listeleme
            "province-list": {
                "prefix": "main",
                "prev": "electricity-service",
                "label": "province-list",
            },
            ## Gün bilgileri
            "date-init": {
                "prefix": "main",
                "prev": "electricity-service",
                "label": "date-init",
            },
            ## Piyasa Katılımcıları
            "market-participants": {
                "prefix": "data",
                "prev": "general-data",
                "label": "market-participants",
            },
            ##Piyasa Katılımcıları Organizasyon
            "market-participants-organization-list": {
                "prefix": "data",
                "prev": "general-data",
                "label": "market-participants-organization-filter-list",
            },
            ##Lisans Türüne Göre Katılımcı Sayısı
            "participant-count-based-upon-license-type": {
                "prefix": "data",
                "prev": "general-data",
                "label": "participant-count-based-upon-license-type",
            },
            ## Aktif Doluluk
            "dams-active-fullness": {
                "prefix": "data",
                "prev": "dams",
                "label": "active-fullness",
            },
            ## Günlük Kot
            "dams-daily-level": {
                "prefix": "data",
                "prev": "dams",
                "label": "daily-kot",
            },
            ## Aktif Hacim
            "dams-active-volume": {
                "prefix": "data",
                "prev": "dams",
                "label": "active-volume",
            },
            ## Günlük Hacim
            "dams-daily-volume": {
                "prefix": "data",
                "prev": "dams",
                "label": "daily-volume",
            },
            ## Havza Listesi
            "basin-list": {
                "prefix": "data",
                "prev": "dams",
                "label": "basin-list",
            },
            ## Baraj Listesi
            "dam-list": {
                "prefix": "data",
                "prev": "dams",
                "label": "dam-list",
            },
            ## Baraj Kot Min-Max
            "dams-level-minmax": {
                "prefix": "data",
                "prev": "dams",
                "label": "dam-kot",
            },
            ## Baraj Hacim Min-Max
            "dams-volume-minmax": {
                "prefix": "data",
                "prev": "dams",
                "label": "dam-volume",
            },
            ## Baraj Debi ve Kurulu Güç
            "dams-info": {
                "prefix": "data",
                "prev": "dams",
                "label": "flow-rate-and-installed-power",
            },
            ## Suyun Enerji Karşılığı
            "dams-water-energy-provision": {
                "prefix": "data",
                "prev": "dams",
                "label": "water-energy-provision",
            },
            # GİP Kontrat Özeti
            "idm-summary": {
                "prefix": "data",
                "prev": "reporting-service",
                "label": "idm-contract-summary",
            },
            # Elektrik Piyasa Hacimleri Fiziksel
            "electricity-market-quantity": {
                "prefix": "data",
                "prev": "reporting-service",
                "label": "electricity-market-volume-physically",
            },
            ## GİP Kontrat Listeleme Servisi
            "idm-contract-list": {
                "prefix": "data",
                "prev": "reporting-service",
                "label": "gip-kontrat",
            },
            ## GİP Teklif Listesi
            "idm-order-history": {
                "prefix": "data",
                "prev": "reporting-service",
                "label": "idm-order-list",
            },
            ## GDDK Dağıtım Liste
            "ra-distribution-list": get_path_template("ra", "distribution-list"),
            ## GDDK Sayaç Okuyan Kurum Liste
            "ra-organization-list": get_path_template("ra", "organization-list"),
            ## GDDK Profil Abone Grubu Liste Servisi
            "ra-spg-list": get_path_template("ra", "subscriber-profile-group-list"),
            ## GDDK Hacim Profil Abone Grubu Liste Servisi
            "ra-vspg-list": get_path_template(
                "ra", "volume-subscriber-profile-group-list"
            ),
            ## GDDK’ya Konu olan Sayaç Sayısı Listeleme Servisi
            "ra-meters": get_path_template(
                "ra", "meter-count-subject-to-retroactive-adjustment"
            ),
            ## GDDK’ya Konu olan Sayaç Hacim Verileri Listeleme Servisi (with Period)
            "ra-meter-volumes-period": get_path_template("ra", "meter-volume"),
            ## GDDK’ya Konu olan Sayaç Hacim Verileri Listeleme Servisi (with Version)
            "ra-meter-volumes-version": get_path_template("ra", "meter-volume"),
            ## GDDK Tutarı
            "ra-sum": get_path_template("ra", "retroactive-adjustment-sum"),
            ## Doğal Gaz Piyasa Katılımcıları
            "ng-participants": get_path_template("ng-general", "market-participant"),
            ## Doğal Gaz Katılımcı Listesi
            "ng-participant-list": get_path_template("ng-general", "participant-list"),
            ## İlave Dengeleyici Bildirimleri Listeleme Servisi
            "ng-balancing-notifications": get_path_template(
                "ng-sgp", "additional-notifications"
            ),
            ## Dengeleme Gazı Fiyatları (DGF) Listeleme Servisi
            "ng-balancing-price": get_path_template("ng-sgp", "balancing-gas-price"),
            ## Bakiye Sıfırlama Tutarı (BAST) Listeleme Servisi
            "ng-bast": get_path_template("ng-sgp", "bast"),
            ## 2 Kodlu İşlemler Listeleme Servisi
            "ng-blue-code-ops": get_path_template("ng-sgp", "blue-code-operation"),
            ## SGP Günlük Eşleşme Miktarı Listeleme Servisi
            "ng-daily-match-qty": get_path_template("ng-sgp", "daily-matched-quantity"),
            ## Günlük Referans Fiyatı (GRF) Listeleme Servisi
            "ng-drp": get_path_template("ng-sgp", "daily-reference-price"),
            ## SGP Günlük İşlem Hacmi Listeleme Servisi
            "ng-daily-trade-volume": get_path_template("ng-sgp", "daily-trade-volume"),
            ## 4 Kodlu İşlemler Listeleme Servisi
            "ng-code-four-ops": get_path_template("ng-sgp", "four-code-operation"),
            ## Geriye Dönük Düzeltme Kalemi (GDDK) Tutarı Listeleme Servisi
            "ng-gddk": get_path_template("ng-sgp", "gddk-amount"),
            ## 1 Kodlu İşlemler Listeleme Servisi
            "ng-green-code-ops": get_path_template("ng-sgp", "green-code-operation"),
            ## GRF Eşleşme Miktarı Listeleme Servisi
            "ng-grp-match-qty": get_path_template("ng-sgp", "grf-match-quantity"),
            ## GRF İşlem Hacmi Listeleme Servisi
            "ng-grp-trade-volume": get_path_template("ng-sgp", "grf-trade-volume"),
            ## SGP Dengesizlik Tutarı Listeleme Servisi
            "ng-imbalance-amount": get_path_template("ng-sgp", "imbalance-amount"),
            ## SGP Dengesizlik Sistem Listeleme Servisi
            "ng-imbalance-system": get_path_template("ng-sgp", "imbalance-system"),
            ## Sgp Son Uzlaştırma Tarihi Servisi
            "ng-latest-settlement-date": get_path_template(
                "ng-sgp", "last-reconciliation-date"
            ),
            ## SGP Toplam Eşleşme Miktarı Listeleme Servisi
            "ng-match-quantity": get_path_template("ng-sgp", "match-quantity"),
            ## 3 Kodlu İşlemler Listeleme Servisi
            "ng-orange-code-ops": get_path_template("ng-sgp", "orange-code-operation"),
            ## Fiziki Gerçekleşme Listeleme Servisi
            "ng-physical-realization": get_path_template(
                "ng-sgp", "physical-realization"
            ),
            ## SGP Fiyatlar Listeleme Servisi
            "ng-spot-prices": get_path_template("ng-sgp", "sgp-price"),
            # get_path_template("ng-sgp", "shippers-imbalance-quantity"),
            ## Dengesizlik Taşıtan Listeleme Servisi
            "ng-shippers-imbalance-quantity": get_path_template(
                "ng-sgp", "shippers-imbalance-quantity"
            ),
            ## Sistem Yönü Listeleme Servisi
            "ng-system-direction": get_path_template("ng-sgp", "system-direction"),
            ## SGP Toplam İşlem Hacmi Listeleme Servisi
            "ng-total-trade-volume": get_path_template("ng-sgp", "total-trade-volume"),
            ## SGP İşlem Akışı Listeleme Servisi
            "ng-transaction-history": get_path_template(
                "ng-sgp", "transaction-history"
            ),
            ## Sanal Gerçekleşme Listeleme Servisi
            "ng-virtual-realization": get_path_template(
                "ng-sgp", "virtual-realization"
            ),
            ## SGP Haftalık Eşleşme Miktarı Listeleme Servisi
            "ng-weekly-matched-quantity": get_path_template(
                "ng-sgp", "weekly-matched-quantity"
            ),
            ## Haftalık Referans Fiyatı (HRF) Listeleme Servisi
            "ng-wrp": get_path_template("ng-sgp", "weekly-ref-price"),
            ## SGP Haftalık İşlem Hacmi Listeleme Servisi
            "ng-weekly-trade-volume": get_path_template(
                "ng-sgp", "weekly-trade-volume"
            ),
            # VGP Kontrat Fiyatları Özeti Listeleme Servisi
            "ng-vgp-contract-price-summary": get_path_template(
                "ng-vgp", "contract-price-summary"
            ),
            # VGP Kontrat Fiyatları Özeti Listeleme Servisi (Period)
            "ng-vgp-contract-price-summary-period": get_path_template(
                "ng-vgp", "contract-price-summary"
            ),
            # VGP Kontrat Fiyatları Özeti Listeleme Servisi (Start End)
            "ng-vgp-contract-price-summary-se": get_path_template(
                "ng-vgp", "contract-price-summary"
            ),
            ## VGP Teslimat Dönemi
            "ng-vgp-delivery-period": get_path_template("ng-vgp", "delivery-period"),
            ## VGP Teslimat Yılı
            "ng-vgp-delivery-year": get_path_template("ng-vgp", "delivery-year"),
            ## VGP Günlük Gösterge Fiyatı Listeleme Servisi
            "ng-vgp-ggf": get_path_template("ng-vgp", "ggf"),
            ## VGP Günlük Gösterge Fiyatı Listeleme Servisi (Period)
            "ng-vgp-ggf-period": get_path_template("ng-vgp", "ggf"),
            ## VGP Günlük Gösterge Fiyatı Listeleme Servisi (Start End)
            "ng-vgp-ggf-se": get_path_template("ng-vgp", "ggf"),
            ## VGP Piyasa Eşleşme Miktarı (1000.Sm³/gün) Listeleme Servisi
            "ng-vgp-matched-quantity": get_path_template("ng-vgp", "matching-quantity"),
            ## VGP Piyasa Eşleşme Miktarı (1000.Sm³/gün) Listeleme Servisi (Period)
            "ng-vgp-matched-quantity-period": get_path_template(
                "ng-vgp", "matching-quantity"
            ),
            ## VGP Piyasa Eşleşme Miktarı (1000.Sm³/gün) Listeleme Servisi (Start End)
            "ng-vgp-matched-quantity-se": get_path_template(
                "ng-vgp", "matching-quantity"
            ),
            ## VGP Açık Pozisyon Miktarı (1000.Sm³/gün) Listeleme Servisi
            "ng-vgp-open-positions": get_path_template("ng-vgp", "open-position"),
            ## VGP Açık Pozisyon Miktarı (1000.Sm³/gün) Listeleme Servisi (Period)
            "ng-vgp-open-positions-period": get_path_template(
                "ng-vgp", "open-position"
            ),
            ## VGP Açık Pozisyon Miktarı (1000.Sm³/gün) Listeleme Servisi (Start End)
            "ng-vgp-open-positions-se": get_path_template("ng-vgp", "open-position"),
            ## VGP Teklif Fiyatları Listeleme Servisi
            "ng-vgp-order-book": get_path_template("ng-vgp", "vgp-offer-price"),
            ## VGP İşlem Akışı Listeleme Servisi
            "ng-vgp-transaction-history": get_path_template(
                "ng-vgp", "vgp-transaction-history"
            ),
            ## VGP İşlem Akışı Listeleme Servisi (Period)
            "ng-vgp-transaction-history-period": get_path_template(
                "ng-vgp", "vgp-transaction-history"
            ),
            ## VGP İşlem Akışı Listeleme Servisi (Start End)
            "ng-vgp-transaction-history-se": get_path_template(
                "ng-vgp", "vgp-transaction-history"
            ),
            ## VGP İşlem Hacmi Listeleme Servisi
            "ng-vgp-transaction-volumes": get_path_template("ng-vgp", "vgp-volume"),
            ## VGP İşlem Hacmi Listeleme Servisi (Period)
            "ng-vgp-transaction-volumes-period": get_path_template(
                "ng-vgp", "vgp-volume"
            ),
            ## VGP İşlem Hacmi Listeleme Servisi (Start End)
            "ng-vgp-transaction-volumes-se": get_path_template("ng-vgp", "vgp-volume"),
            ## Kapasite Nokta Servisi
            "ng-tr-capacity-point": get_path_template("ng-tr", "capacity-point"),
            ## Günlük Gerçekleşme Miktarı Listeleme Servisi
            "ng-tr-daily-transmission": get_path_template(
                "ng-tr", "daily-actualization-amount"
            ),
            ## Gün Öncesi (UDN) Listeleme Servisi
            "ng-tr-day-ahead": get_path_template("ng-tr", "day-ahead"),
            ## Gün Sonu (UDN) Listeleme Servisi
            "ng-tr-day-end": get_path_template("ng-tr", "day-end"),
            ## Taşıma Giriş Miktarı Bildirimi (TMB) Listeleme Servisi
            "ng-tr-entry-nomination": get_path_template("ng-tr", "entry-nomination"),
            ## Taşıma Çıkış Miktarı Bildirimi (TMB) Listeleme Servisi
            "ng-tr-exit-nomination": get_path_template("ng-tr", "exit-nomination"),
            ## Maks Giriş Kapasite Miktarı Listeleme Servisi
            "ng-tr-max-entry-amount": get_path_template("ng-tr", "max-entry-amount"),
            ## Maks Çıkış Kapasite Miktarı Listeleme Servisi
            "ng-tr-max-exit-amount": get_path_template("ng-tr", "max-exit-amount"),
            ## Fiili Gerçekleşme Giriş Kapasite Miktarı Listeleme Servisi
            "ng-tr-actual-entry-amount": get_path_template(
                "ng-tr", "realization-entry-amount"
            ),
            ## Fiili Gerçekleşme Çıkış Kapasite Miktarı Listeleme Servisi
            "ng-tr-actual-exit-amount": get_path_template(
                "ng-tr", "realization-exit-amount"
            ),
            ## Rezerve Giriş Kapasite Miktarı Listeleme Servisi
            "ng-tr-reserved-entry-amount": get_path_template(
                "ng-tr", "rezerve-entry-amount"
            ),
            ## Rezerve Çıkış Kapasite Miktarı Listeleme Servisi
            "ng-tr-reserved-exit-amount": get_path_template(
                "ng-tr", "rezerve-exit-amount"
            ),
            ## Stok Miktarı Listeleme Servisi
            "ng-tr-stock-amount": get_path_template("ng-tr", "stock-amount"),
            ## Depolama Tesisi Listeleme Servisi
            "ng-tr-storage-facility-list": get_path_template(
                "ng-tr", "storage-facility"
            ),
            ## Transfer Listeleme Servisi
            "ng-tr-bilateral-transfer": get_path_template("ng-tr", "transfer"),
            ## Menu Tree
            "menu": {
                "prefix": "menu",
                "prev": "electricity-service",
                "label": "get-menu-tree",
            },
            "page-settings": {
                "prefix": "menu",
                "prev": "electricity-service",
                "label": "get-page-settings",
            },
            ## YEK-G İkili Anlaşma Miktarları Listeleme Servisi
            "yek-g-bilateral-contracts": get_path_template(
                "yek-g", "bilateral-contract-list"
            ),
            ## YEK-G İtfa İşlem Miktarları Listeleme Servisi
            "yek-g-redemptions": get_path_template("yek-g", "cancelation-quantity"),
            ## İlga edilen YEK-G Belge Miktarı Listeleme Servisi
            "yek-g-expirations": get_path_template("yek-g", "expiry-quantity"),
            ## İhraç edilen YEK-G Belge Miktarı Listeleme Servisi
            "yek-g-issued": get_path_template("yek-g", "exported-document-quantity"),
            ## YEK-G Org. Piyasa Alış/Satış Teklif Miktarı
            "yek-g-order-quantity": get_path_template(
                "yek-g", "market-bid-ask-quantity"
            ),
            ## YEK-G Min-Max Eşleşme Fiyatları Listeleme
            "yek-g-match-price-minmax": get_path_template(
                "yek-g", "min-max-match-amount-list"
            ),
            ## YEK-G Org. Piyasa İşlem Hacmi Listeleme Servisi
            "yek-g-trade-volume": get_path_template("yek-g", "trading-volume"),
            ## YEK-G Ağırlıklı Ortalama Fiyat Listeleme Servisi
            "yek-g-wap": get_path_template("yek-g", "weighted-average-price"),
            ## İptal edilen YEK-G Belge Miktarı Listeleme Servisi
            "yek-g-withdrawals": get_path_template("yek-g", "withdrawal-quantity"),
            ## Org. YEK-G Piyasa Eşleşme Miktarları Listeleme Servisi
            "yek-g-match-quantity": get_path_template(
                "yek-g", "yekg-matching-quantity"
            ),
            ## VEP Kontrat Fiyatları Özet Listeleme Servisi
            "vep-contract-price-summary": get_path_template(
                "vep", "contract-price-summary"
            ),
            ## VEP Teslimat Dönemi Listeleme Servisi
            "vep-delivery-period-list": get_path_template(
                "vep", "delivery-period-list"
            ),
            ## VEP Teslimat Yılı Listeme Servisi
            "vep-delivery-year-list": get_path_template("vep", "delivery-year-list"),
            ## VEP Günlük Gösterge Fiyatı Listeleme Servisi
            "vep-ggf": get_path_template("vep", "ggf"),
            ## VEP GGF Teslimat Dönemi Listesi
            "vep-ggf-period": get_path_template("vep", "ggf-delivery-period-list"),
            ## VEP Yük Tipi Listeleme Servisi
            "vep-load-types": get_path_template("vep", "load-type-list"),
            ## VEP Teklif Fiyatları Listeleme Servisi
            "vep-price-summaries": get_path_template("vep", "offer-price"),
            ## VEP Açık Pozisyon Listeleme Servisi
            "vep-open-positions": get_path_template("vep", "open-position"),
            ## VEP İşlem Hacmi Listeleme Servisi
            "vep-trade-volume": get_path_template("vep", "pfm-trade-value"),
            ## VEP İşlem Akışı Listeleme Servisi
            "vep-transaction-history": get_path_template(
                "vep", "pfm-transaction-history"
            ),
            ## VEP İşlem Akışı Teslimat Dönemi Listesi
            "vep-transaction-history-periods": get_path_template(
                "vep", "th-delivery-period-list"
            ),
            ## VEP Eşleşme Miktarı Listeleme Servisi
            "vep-matching-quantity": get_path_template("vep", "vep-matching-quantity"),
            ## İl, İlçe ST Adedi Listeleme Servisi
            "eligible-consumer-count-detail": get_path_template(
                "cons", "eligible-consumer-count"
            ),
            ## Serbest Tüketici Tüketim Miktarı Listeleme
            "eligible-consumer-quantity": get_path_template(
                "cons", "eligible-consumer-quantity"
            ),
            ## Dağıtım Şirketlerinin Alındığı Servis
            "get-distribution-companies": get_path_template(
                "cons", "get-distribution-companies"
            ),
            ## Ana Tarife Grubu
            "main-tariff-group-list": get_path_template(
                "cons", "main-tariff-group-list"
            ),
            ## Aylık Endeks Listeleme Servisi
            "monthly-index": get_path_template("cons", "monthly-index"),
            ## Çarpan Değeri Listeleme Servisi
            "multiple-factor": get_path_template("cons", "multiple-factor"),
            ## Dağıtım Firmaları Listeleme Servisi
            "mf-distribution": get_path_template(
                "cons", "multiple-factor-distribution"
            ),
            ## Sayaç Okuma Tipi Listeleme Servisi
            "mf-meter-reading-type": get_path_template(
                "cons", "multiple-factor-meter-reading-type"
            ),
            ## Profil Abone Grubu Listeleme Servisi
            "mf-profile-group": get_path_template(
                "cons", "multiple-factor-profile-group"
            ),
            ## Yüzdesel Tüketim Bilgileri Listeleme Servisi
            "percentage-consumption-info": get_path_template(
                "cons", "percentage-consumption-info"
            ),
            ## Planlı Kesinti Bilgisi Listeleme Servisi
            "planned-outages": get_path_template("cons", "planned-power-outage-info"),
            ## (İl, İlçe ST Adedi) Profil Abone Grubu Listeleme Servisi
            "elig-profile-groups": get_path_template(
                "cons", "profile-subscription-group-list"
            ),
            ## Serbest Tüketici Adedi Listeleme Servisi
            "eligible-consumer-count": get_path_template("cons", "st-adedi"),
            ## Plansız Kesinti Bilgisi Listeleme Servisi
            "unplanned-outages": get_path_template(
                "cons", "unplanned-power-outage-info"
            ),
        },
        ## category
        "idm": {"prev": "markets"},
        "dam": {"prev": "markets"},
        "bpm": {"prev": "markets"},
        "sgp": {"prev": "markets"},
        "vgp": {"prev": "markets"},
        "yek-g": {"prev": "markets"},
        "pfm": {"prev": "markets"},
        "bilateral-contracts": {"prev": "markets"},
        "general-data": {"prev": "markets"},
        "imbalance": {"prev": "markets"},
        "retroactive-adjustment": {"prev": "markets"},
        "ancillary-services": {"prev": "markets"},
        "dams": {"prev": "electricity-service"},
        "markets": {"prev": "electricity-service"},
        "generation": {"prev": "electricity-service"},
        "consumption": {"prev": "electricity-service"},
        "renewables": {"prev": "electricity-service"},
        "transmission": {"prev": "electricity-service"},
        # "transmission-ng": {"label": "transmission"},
        #### services
        "electricity-service": {"next": "version"},
        "reporting-service": {"next": "version"},
        "version": {"label": "v1"},
    }

    if just_call_keys:
        return list(path_map["call"].keys())
    else:
        call_d = path_map.pop("call")
        return {**path_map, **call_d}


def get_total_path(key: str, join_path: bool = True):
    d = get_path_map().get(key, None)
    redirect_key = None if d is None else d.get("redirect", None)
    if redirect_key is not None:
        d = get_path_map().get(redirect_key, None)

    if d is not None:
        total_path = [d.get("label", key)]
        if d.get("prefix", None) is not None:
            total_path = [d["prefix"]] + total_path
        if d.get("suffix", None) is not None:
            total_path = total_path + [d["suffix"]]

        if d.get("prev", None) is not None:
            total_path = get_total_path(key=d["prev"], join_path=False) + total_path
        if d.get("next", None) is not None:
            total_path += get_total_path(key=d["next"], join_path=False)

        full_path = "/".join(total_path) if join_path else total_path
        root_path = d.get("root", None)
        if root_path is not None:
            full_path = re.sub("electricity-service", root_path + "-service", full_path)
        return full_path
    else:
        raise Exception("Key not found in path map.")


def get_call_method(key):
    """
    Get the call method for a given key. If the key is in the list of keys that require GET method, return GET, else return POST.
    """
    get_methods = [
        "date-init",
        "interim-mcp-status",
        "market-participants-organization-list",
        "pp-list",
        "uevm-pp-list",
        "region-list",
        "mms-message-type-list",
        "mms-region-list",
        "intl-direction-list",
        "intl-capacity-demand-direction-list",
        "basin-list",
        "distribution-region-list",
        "province-list",
        "profile-group-list",
        "ra-distribution-list",
        "ra-organization-list",
        "ra-spg-list",
        "ra-vspg-list",
        "ng-participant-list",
        "ng-latest-settlement-date",
        "ng-vgp-delivery-period",
        "ng-vgp-delivery-year",
        "ng-tr-storage-facility-list",
        "menu",
        "get-distribution-companies",
        "main-tariff-group-list",
        "mf-meter-reading-type",
    ]

    if key in get_methods:
        return "GET"
    else:
        return "POST"


def get_derived_calls():
    """
    These calls are specific versions with parameter combinations that are used in the API
    """

    l = [
        "ng-vgp-contract-price-summary-period",
        "ng-vgp-ggf-period",
        "ng-vgp-matched-quantity-period",
        "ng-vgp-open-positions-period",
        "ng-vgp-transaction-history-period",
        "ng-vgp-transaction-volumes-period",
        "ng-vgp-contract-price-summary-se",
        "ng-vgp-ggf-se",
        "ng-vgp-matched-quantity-se",
        "ng-vgp-open-positions-se",
        "ng-vgp-transaction-history-se",
        "ng-vgp-transaction-volumes-se",
        "ra-meter-volumes-period",
        "ra-meter-volumes-version",
    ]

    return l



def get_param_label(key):
    d = {
        "start_date": {
            "label": "startDate",
        },
        "end_date": {
            "label": "endDate",
        },
        "date_time": {"label": "date"},
        "date": {"label": "date"},
        "period": {"label": "period"},
        "org_id": {"label": "organizationId"},
        "uevcb_id": {"label": "uevcbId"},
        "imb_org_id": {"label": "organizationId"},
        "region": {"label": "region"},
        "region_id": {"label": "regionId"},
        "pp_id": {"label": "powerPlantId"},
        # "pp_id2": {
        #     "label": "powerplantId"
        # },  ## Because all the other calls use powerPlantId and uevm uses powerplantId (lowercase p)
        # UPDATE: Fixed somewhere else
        "year": {"label": "year"},
        "price_type": {"label": "priceType"},
        "order_type": {"label": "orderType"},
        "message_type_id": {"label": "mesajTipId"},
        "intl_direction": {"label": "direction"},
        "province_id": {"label": "provinceId"},
        "uevcb_name": {"label": "uevcbName"},
        "basin_name": {"label": "basinName"},
        "dam_name": {"label": "damName"},
        "idm_contract_id": {"label": "contractId"},
        "se_date": {"label": ["startDate", "endDate"]},  ## Start-end date
        "dist_org_id": {
            "label": "distrubutionOrganization"
        },  ## TYPO is intentional (same as api)
        "province_id": {"label": "provinceId"},
        "profile_group_id": {"label": "profileGroupId"},
        "distribution_id": {"label": "distributionId"},
        "spg_name": {"label": "subscriberProfileGroupName"},
        "mr_org_id": {"label": "meterReadOrgId"},
        "period_start_date": {"label": "periodStartDate"},
        "period_end_date": {"label": "periodEndDate"},
        "version_start_date": {"label": "versionStartDate"},
        "version_end_date": {"label": "versionEndDate"},
        "is_txn_period": {"label": "isTransactionPeriod"},
        "delivery_year": {"label": "deliveryYear"},
        "delivery_period": {"label": "deliveryPeriod"},
        "point_type": {"label": "pointType"},
        "storage_facility_id": {"label": "storageFacilityId"},
        "point_id": {"label": "pointId"},
        "menu_id": {"label": "menuId"},
        "load_type": {"label": "loadType"},
        "tariff_group_id": {"label": "groupId"},
        "mr_type": {"label": "meterReadingType"},
        "subscriber_pg": {"label": "subscriberProfileGroup"},
        "province_id": {"label": "provinceId"},
        "dist_company_id": {"label": "distributionCompanyId"},
        "district_name": {"label": "districtName"},
        "pg_name": {"label": "profileGroupName"},
    }
    return d.get(key, key)


def get_required_parameters(key, return_mapping=False, mapping_only_keys=False):
    d = {
        "mcp": ["start_date", "end_date"],
        "interim-mcp": ["start_date", "end_date"],
        "interim-mcp-status": [],
        "date-init": [],
        "smp": ["start_date", "end_date"],
        "smp-dir": ["start_date", "end_date"],
        "bpm-up": ["start_date", "end_date"],
        "bpm-down": ["start_date", "end_date"],
        "dam-volume": ["start_date", "end_date"],
        "pi-offer": ["start_date", "end_date"],
        "pi-bid": ["start_date", "end_date"],
        "dam-bid": ["start_date", "end_date"],
        "dam-offer": ["start_date", "end_date"],
        "supply-demand": ["date_time"],
        "dam-clearing": ["start_date", "end_date"],
        "dam-clearing-org-list": ["period"],
        "dam-block-bid": ["start_date", "end_date"],
        "dam-block-offer": ["start_date", "end_date"],
        "dam-flexible-matching": ["start_date", "end_date"],
        "dam-flexible-bid": ["start_date", "end_date"],
        "dam-flexible-offer": ["start_date", "end_date"],
        "dam-diff": ["start_date", "end_date"],
        "wap": ["start_date", "end_date"],
        "idm-ob-qty": ["start_date", "end_date"],
        "idm-qty": ["start_date", "end_date"],
        "idm-mm-offer": ["start_date", "end_date"],
        "idm-mm-bid": ["start_date", "end_date"],
        "idm-mm-matching": ["start_date", "end_date"],
        "idm-volume": ["start_date", "end_date"],
        "idm-log": ["start_date", "end_date"],
        "bi-long": ["start_date", "end_date"],
        "bi-short": ["start_date", "end_date"],
        "bi-euas": ["start_date", "end_date"],
        "imb-qty": ["start_date", "end_date"],
        "imb-vol": ["start_date", "end_date"],
        "imb-qty-g": ["start_date", "end_date"],
        "imb-org-list": ["start_date", "end_date"],
        "mcp-smp-imb": ["start_date", "end_date"],
        "bpm-orders-w-avg": ["date"],
        "bpm-orders": ["date"],
        "market-participants": [],
        "market-participants-organization-list": [],
        "kgup": ["start_date", "end_date", "region"],
        "kgup-v1": ["start_date", "end_date", "region"],
        "kudup": ["start_date", "end_date", "region"],
        "eak": ["start_date", "end_date", "region"],
        "gen-org": ["start_date", "end_date"],
        "gen-uevcb": ["org_id", "start_date"],
        "rt-gen": ["start_date", "end_date"],
        "uevm": ["start_date", "end_date"],
        "ren-uevm": ["start_date", "end_date"],
        "uecm": ["start_date", "end_date"],
        "su-uecm": ["start_date", "end_date"],
        "st-uecm": ["period"],
        "lic-pp-list": ["start_date", "end_date"],
        "load-plan": ["start_date", "end_date"],
        "rt-cons": ["start_date", "end_date"],
        "pp-list": [],
        "uevm-pp-list": [],
        "ren-pp-list": ["period"],
        "ren-rt-gen": ["start_date", "end_date"],
        "ren-ul-gen": ["start_date", "end_date"],
        "ren-ul-cost": ["start_date", "end_date"],
        "ren-unit-cost": ["start_date", "end_date"],
        "ren-income": ["start_date", "end_date"],
        "ren-lic-cost": ["start_date", "end_date"],
        "ren-total-cost": ["start_date", "end_date"],
        "ren-capacity": ["period"],
        "wind-forecast": ["start_date", "end_date"],
        "ren-participant-list": ["year"],
        "zero-balance": ["start_date", "end_date"],
        "iskk": ["start_date", "end_date"],
        "congestion-cost": ["start_date", "end_date", "price_type", "order_type"],
        "eic-x-org-list": ["period"],
        "eic-w-org-list": ["period"],
        "eic-w-uevcb-list": ["uevcb_name", "period"],
        "mms": ["start_date", "end_date", "region_id"],
        "region-list": [],
        "mms-pp-list": ["start_date"],
        "mms-uevcb-list": ["start_date", "pp_id"],
        "mms-message-type-list": [],
        "mms-region-list": [],
        "international-line-events": ["start_date", "end_date"],
        "tcat-pre-year-forecast": ["start_date", "end_date"],
        "tcat-pre-month-forecast": ["start_date", "end_date"],
        "line-capacities": ["start_date", "end_date", "intl_direction"],
        "intl-direction-list": [],
        "intl-capacity-demand-direction-list": [],
        "capacity-demand": ["start_date", "end_date", "intl_direction"],
        "nominal-capacity": ["start_date", "end_date"],
        "dams-active-fullness": [],
        "dams-daily-level": [],
        "dams-active-volume": [],
        "dams-daily-volume": [],
        "basin-list": [],
        "dam-list": [],
        "dams-level-minmax": [],
        "dams-volume-minmax": [],
        "dams-info": [],
        "dams-water-energy-provision": [],
        "idm-summary": ["start_date", "end_date"],
        "electricity-market-quantity": ["start_date", "end_date"],
        "idm-contract-list": ["se_date"],
        "idm-order-history": ["se_date", "idm_contract_id"],
        "participant-count-based-upon-license-type": ["start_date"],
        "anc-pf-qty": ["start_date", "end_date"],
        "anc-pfk": ["start_date", "end_date"],
        "anc-sf-qty": ["start_date", "end_date"],
        "anc-sfk": ["start_date", "end_date"],
        "distribution-region-list": [],
        "long-term-demand-forecast": [],
        "consumption-breakdown": [
            "period"
        ],  ## This is actually end of month period in the api but it is ok
        "province-list": [],
        "district-list": ["province_id"],
        "profile-group-list": [],
        "consumer-breakdown": ["period"],
        "ra-distribution-list": [],
        "ra-organization-list": [],
        "ra-spg-list": [],
        "ra-vspg-list": [],
        "ra-meters": ["distribution_id", "start_date", "end_date"],
        "ra-meter-volumes-period": [
            "mr_org_id",
            "period_start_date",
            "period_end_date",
        ],
        "ra-meter-volumes-version": [
            "mr_org_id",
            "version_start_date",
            "version_end_date",
        ],
        "ra-sum": ["start_date", "end_date"],
        "ng-participants": [],
        "ng-participant-list": [],
        "ng-balancing-notifications": ["start_date", "end_date"],
        "ng-balancing-price": ["start_date", "end_date"],
        "ng-bast": ["period"],
        "ng-blue-code-ops": ["start_date", "end_date"],
        "ng-daily-match-qty": ["start_date", "end_date"],
        "ng-drp": ["start_date", "end_date"],
        "ng-daily-trade-volume": ["start_date", "end_date"],
        "ng-code-four-ops": ["start_date", "end_date"],
        "ng-gddk": ["start_date", "end_date"],
        "ng-green-code-ops": ["start_date", "end_date"],
        "ng-grp-match-qty": ["start_date", "end_date"],
        "ng-grp-trade-volume": ["start_date", "end_date"],
        "ng-imbalance-amount": ["period"],
        "ng-imbalance-system": ["start_date", "end_date"],
        "ng-latest-settlement-date": [],
        "ng-match-quantity": ["start_date", "end_date"],
        "ng-orange-code-ops": ["start_date", "end_date"],
        "ng-physical-realization": ["start_date", "end_date"],
        "ng-spot-prices": ["start_date", "end_date"],
        "ng-shippers-imbalance-quantity": ["period"],
        "ng-system-direction": ["start_date", "end_date"],
        "ng-total-trade-volume": ["start_date", "end_date"],
        "ng-transaction-history": ["start_date", "end_date"],
        "ng-virtual-realization": ["start_date", "end_date"],
        "ng-weekly-matched-quantity": ["start_date", "end_date"],
        "ng-wrp": ["start_date", "end_date"],
        "ng-weekly-matched-quantity": ["start_date", "end_date"],
        "ng-weekly-trade-volume": ["start_date", "end_date"],
        "ng-vgp-contract-price-summary": ["is_txn_period"],
        "ng-vgp-contract-price-summary-period": [
            "is_txn_period",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-contract-price-summary-se": ["is_txn_period", "start_date", "end_date"],
        "ng-vgp-delivery-period": [],
        "ng-vgp-delivery-year": [],
        "ng-vgp-ggf": ["is_txn_period"],
        "ng-vgp-ggf-period": [
            "is_txn_period",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-ggf-se": ["is_txn_period", "start_date", "end_date"],
        "ng-vgp-matched-quantity": ["is_txn_period"],
        "ng-vgp-matched-quantity-period": [
            "is_txn_period",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-matched-quantity-se": ["is_txn_period", "start_date", "end_date"],
        "ng-vgp-open-positions": ["is_txn_period"],
        "ng-vgp-open-positions-period": [
            "is_txn_period",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-open-positions-se": ["is_txn_period", "start_date", "end_date"],
        "ng-vgp-order-book": [],
        "ng-vgp-transaction-history": ["is_txn_period"],
        "ng-vgp-transaction-history-period": [
            "is_txn_period",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-transaction-history-se": ["is_txn_period", "start_date", "end_date"],
        "ng-vgp-transaction-volumes": ["is_txn_period"],
        "ng-vgp-transaction-volumes-period": [
            "is_txn_period",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-transaction-volumes-se": ["is_txn_period", "start_date", "end_date"],
        "ng-tr-capacity-point": ["start_date", "end_date", "point_type"],
        "ng-tr-daily-transmission": ["start_date", "end_date"],
        "ng-tr-day-ahead": ["start_date", "end_date"],
        "ng-tr-day-end": ["start_date", "end_date"],
        "ng-tr-entry-nomination": ["start_date", "end_date"],
        "ng-tr-exit-nomination": ["start_date", "end_date"],
        "ng-tr-max-entry-amount": ["start_date", "end_date"],
        "ng-tr-max-exit-amount": ["start_date", "end_date"],
        "ng-tr-actual-entry-amount": ["start_date", "end_date"],
        "ng-tr-actual-exit-amount": ["start_date", "end_date"],
        "ng-tr-reserved-entry-amount": ["start_date", "end_date"],
        "ng-tr-reserved-exit-amount": ["start_date", "end_date"],
        "ng-tr-stock-amount": ["start_date", "end_date"],
        "ng-tr-bilateral-transfer": ["start_date", "end_date"],
        "ng-tr-storage-facility-list": [],
        "menu": [],
        "page-settings": ["menu_id"],
        "yek-g-bilateral-contracts": ["start_date", "end_date"],
        "yek-g-redemptions": ["start_date", "end_date"],
        "yek-g-expirations": ["start_date", "end_date"],
        "yek-g-issued": ["start_date", "end_date"],
        "yek-g-match-price-minmax": ["start_date", "end_date"],
        "yek-g-order-quantity": ["start_date", "end_date"],
        "yek-g-trade-volume": ["start_date", "end_date"],
        "yek-g-wap": ["start_date", "end_date"],
        "yek-g-withdrawals": ["start_date", "end_date"],
        "yek-g-match-quantity": ["start_date", "end_date"],
        "vep-contract-price-summary": ["start_date", "end_date"],
        "vep-delivery-period-list": ["start_date", "end_date"],
        "vep-delivery-year-list": ["start_date", "end_date"],
        "vep-ggf": ["start_date", "end_date"],
        "vep-ggf-period": ["start_date", "end_date"],
        "vep-load-types": ["start_date", "end_date"],
        "vep-price-summaries": [],
        "vep-open-positions": ["start_date", "end_date"],
        "vep-trade-volume": ["start_date", "end_date"],
        "vep-transaction-history": ["start_date", "end_date"],
        "vep-transaction-history-periods": ["start_date", "end_date"],
        "vep-matching-quantity": ["start_date", "end_date"],
        "eligible-consumer-count-detail": ["period"],
        "get-distribution-companies": [],
        "main-tariff-group-list": [],
        "monthly-index": ["start_date", "end_date", "tariff_group_id"],
        "multiple-factor": [
            "period",
            "mr_type",
            "distribution_id",
            "subscriber_pg",
        ],
        "mf-distribution": ["period"],
        "mf-meter-reading-type": [],
        "mf-profile-group": ["period", "distribution_id"],
        "percentage-consumption-info": ["period", "province_id"],
        "planned-outages": ["period"],
        "elig-profile-groups": ["period"],
        "eligible-consumer-count": ["start_date", "end_date"],
        "unplanned-outages": ["period"],
    }

    if return_mapping:
        if mapping_only_keys:
            return list(d.keys())
        return d

    ## UPDATE: As a precaution every call should have an input parameter
    return d.get(key, [])


def get_optional_parameters(key):
    d = {
        "dam-clearing": ["org_id"],
        "bi-long": ["org_id"],
        "bi-short": ["org_id"],
        "imb-qty-g": ["imb_org_id"],
        "market-participants": ["org_id"],
        "kgup": ["org_id", "uevcb_id"],
        "kgup-v1": ["org_id", "uevcb_id"],
        "kudup": ["org_id", "uevcb_id"],
        "eak": ["org_id", "uevcb_id"],
        "rt-gen": ["pp_id"],
        "ren-rt-gen": ["pp_id"],
        "uevm": ["pp_id"],
        "eic-x-org-list": ["org_id"],
        "eic-w-org-list": ["org_id"],
        "eic-w-uevcb-list": ["province_id"],
        "mms": ["org_id", "uevcb_id", "pp_id", "message_type_id"],
        "mms-pp-list": ["org_id"],
        "dams-active-fullness": ["basin_name", "dam_name"],
        "dams-daily-level": ["basin_name", "dam_name"],
        "dams-active-volume": ["basin_name", "dam_name"],
        "dams-daily-volume": ["basin_name", "dam_name"],
        "dam-list": ["basin_name"],
        "dams-level-minmax": ["basin_name", "dam_name"],
        "dams-volume-minmax": ["basin_name", "dam_name"],
        "dams-info": ["basin_name", "dam_name"],
        "dams-water-energy-provision": ["basin_name", "dam_name"],
        "long-term-demand-forecast": ["dist_org_id"],
        "consumption-breakdown": ["province_id", "profile_group_id"],
        "consumer-breakdown": ["province_id", "profile_group_id"],
        "idm-qty": ["org_id"],
        "ra-meters": ["ra_spg_name"],
        "ra-meter-volume-period": [
            "ra_spg_name",
            "version_start_date",
            "version_end_date",
        ],
        "ra-meter-volume-version": [
            "ra_spg_name",
            "period_start_date",
            "period_end_date",
        ],
        "ng-participants": ["org_id"],
        "ng-vgp-contract-price-summary": [
            "start_date",
            "end_date",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-ggf": [
            "start_date",
            "end_date",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-matched-quantity": [
            "start_date",
            "end_date",
            "delivery_period",
            "delivery_year",
        ],
        "ng-vgp-open-positions": [
            "start_date",
            "end_date",
            "delivery_period",
            "delivery_year",
        ],
        "ng-tr-daily-transmission": ["storage_facility_id"],
        "ng-tr-max-entry-amount": ["point_id"],
        "ng-tr-max-exit-amount": ["point_id"],
        "ng-tr-actual-entry-amount": ["point_id"],
        "ng-tr-actual-exit-amount": ["point_id"],
        "ng-tr-reserved-entry-amount": ["point_id"],
        "ng-tr-reserved-exit-amount": ["point_id"],
        "vep-contract-price-summary": ["load_type", "year", "delivery_period"],
        "vep-ggf": ["load_type", "year", "delivery_period"],
        "vep-open-positions": ["load_type", "year", "delivery_period"],
        "vep-trade-volume": ["load_type", "year", "delivery_period"],
        "vep-transaction-history": ["load_type", "year", "delivery_period"],
        "vep-matching-quantity": ["load_type", "year", "delivery_period"],
        "percentage-consumption-info": ["province_id"],
        "planned-outages": ["province_id", "dist_company_id"],
        "elig-profile-groups": ["district_name", "province_id"],
        "unplanned-outages": ["province_id", "dist_company_id"],
        "eligible-consumer-count": ["district_name", "pg_name", "province_id"],
    }

    return d.get(key, [])



import os
import subprocess


def run_app(username: str, password: str, port: int | None = None):
    """
    Launch the Streamlit app programmatically.
    """
    # Get the absolute path of this script
    script_path = os.path.abspath(__file__)

    os.environ["EPTR_USERNAME"] = username
    os.environ["EPTR_PASSWORD"] = password

    # Run Streamlit CLI command to start the app
    run_l = ["streamlit", "run", script_path]
    if port is not None:
        run_l.extend(["--server.port", str(port)])
    try:
        subprocess.run(run_l, check=True)
    except FileNotFoundError:
        raise RuntimeError(
            "Streamlit is not installed or not found in the PATH. "
            "Please ensure Streamlit is installed in your environment."
        )
    except subprocess.CalledProcessError as e:
        raise RuntimeError(f"Failed to run the Streamlit app: {e}")


if __name__ == "__main__":

    import streamlit as st
    from eptr2 import EPTR2
    from eptr2.mapping import get_call_help, get_help_d
    from datetime import datetime, timedelta
    from eptr2.mapping.processing import get_postprocess_function

    username = os.getenv("EPTR_USERNAME")
    password = os.getenv("EPTR_PASSWORD")

    st.set_page_config(
        page_title="eptr2 Demo",
        page_icon="👩‍💻",
        layout="centered",
        initial_sidebar_state="auto",
        menu_items=None,
    )

    tra_map = {
        "fm": {
            "tr": "[eptr2](https://www.pypi.org/project/eptr2) Python paketini kullanarak Şeffaflık 2.0 üzerinden istediğiniz API'yi aşağıdaki kodları kullanarak çağırabilirsiniz.",
            "en": "You can call any API on Transparency 2.0 using the [eptr2](https://www.pypi.org/project/eptr2) Python package with the following code.",
        },
        "fw": {
            "tr": "Demo bütün APIler için örnek çağırma yapılamayabilir. İyileştirme çalışmalarımız devam etmektedir.",
            "en": "A sample call may not be made for all APIs. Our improvement works continue.",
        },
    }

    ss = st.session_state
    ss["lang"] = ss.get("lang", "tr")

    d = get_help_d()
    # x = get_call_help()
    d2 = {v["title"][ss["lang"]]: {"key": k, **v} for k, v in d.items()}

    ss["eptr"] = ss.get(
        "eptr",
        EPTR2(
            username=username,
            password=password,
            postprocess=False,
        ),
    )
    all_calls = ss["eptr"].get_available_calls()
    missing_calls = [k for k in all_calls if k not in d.keys()]

    default_values = {
        "start_date": (datetime.now() - timedelta(days=1)).strftime("%Y-%m-%d"),
        "end_date": (datetime.now() - timedelta(days=1)).strftime("%Y-%m-%d"),
        "date": (datetime.now() - timedelta(days=1)).strftime("%Y-%m-%d"),
        "se_date": (datetime.now() - timedelta(days=1)).strftime("%Y-%m-%d"),
        "date_time": (datetime.now() - timedelta(days=1)).strftime(
            "%Y-%m-%dT%H:00:00+03:00"
        ),
        "period": (datetime.now() - timedelta(days=1))
        .replace(day=1)
        .strftime("%Y-%m-%d"),
        "org_id": "294",
        "uevcb_id": "3205891",
        "pp_id": "2800",  ## Real Time generation
        "idm_contract_id": "2066419679",  # PH23050120
        "year": "2021",
        "intl_direction": "TRGR",
        "uevcb_name": "AFY",
    }

    ss["call_data"] = ss.get("call_data", {})

    def call_code(help_d, key, just_body_params=False):
        req_params = help_d["required_body_params"]

        opt_params = help_d["optional_body_params"]
        req_params = req_params + opt_params

        body_param_d = ", ".join(
            [x + "=" + '"' + default_values.get(x, "") + '"' for x in req_params]
        )

        if just_body_params:
            bod_params = {
                x: default_values.get(x, "")
                for x in req_params
                if x in default_values.keys()
            }
            return bod_params

        txt = f"""from eptr2 import EPTR2
    eptr = EPTR2(username="YOUR_EPTR_USERNAME", password="YOUR_EPTR_PASSWORD")

    eptr.call("{key}", {body_param_d})
    """

        return txt

    st.title("EPTR2 Demo")
    if st.toggle("🇬🇧 English", value=False):
        if ss["lang"] == "tr":
            ss["lang"] = "en"
            st.rerun()
    elif ss["lang"] == "en":
        ss["lang"] = "tr"
        st.rerun()

    st.markdown(
        "![PyPI - Version](https://img.shields.io/pypi/v/eptr2) ![PyPI - Downloads](https://img.shields.io/pypi/dm/eptr2)"
    )
    st.markdown(tra_map["fm"][ss["lang"]])
    st.warning(tra_map["fw"][ss["lang"]])
    col1, col2, col3, col4 = st.columns([3, 3, 3, 3])

    with col1:
        st.link_button(
            "✉️ İletişim" if ss["lang"] == "tr" else "✉️ Contact",
            "https://robokami.com/#iletisim",
            type="primary",
            use_container_width=True,
        )
    with col2:
        try:
            seffaflik_url = ss["call_data"]["help"].get(
                "url", "https://seffaflik.epias.com.tr/transparency"
            )
        except:
            seffaflik_url = "https://seffaflik.epias.com.tr/transparency"

        st.link_button(
            "⚡️ EPİAŞ Şeffaflık" if ss["lang"] == "tr" else "⚡️ EPIAS TP",
            url=seffaflik_url,
            type="secondary",
            use_container_width=True,
        )
    with col3:
        st.link_button(
            "🔎 Github",
            "https://www.github.com/Tideseed/eptr2",
            use_container_width=True,
        )
    with col4:
        st.link_button(
            "🐍 PyPI",
            "https://www.pypi.org/project/eptr2",
            use_container_width=True,
        )

    st.selectbox(
        "Veri seti seçin" if ss["lang"] == "tr" else "Choose a data set",
        list(d2.keys()) + missing_calls,
        key="eptr2_call",
    )

    if ss.get("eptr2_call", None) is not None:
        ss["call_data"] = get_call_help(
            d2[ss["eptr2_call"]]["key"]
            if d2.get(ss["eptr2_call"], None) is not None
            else ss["eptr2_call"]
        )
        st.subheader(
            ss["call_data"]["help"]["title"][ss["lang"]]
            if d2.get(ss["eptr2_call"], None) is not None
            else "_(başlık yok)_"
        )
        st.markdown(
            ss["call_data"]["help"]["desc"][ss["lang"]]
            if d2.get(ss["eptr2_call"], None) is not None
            else "_(açıklama yok)_"
        )

        st.subheader("Örnek Kullanım" if ss["lang"] == "tr" else "Example Usage")

        keyval = (
            d2[ss["eptr2_call"]]["key"]
            if d2.get(ss["eptr2_call"], None) is not None
            else ss["eptr2_call"]
        )
        st.code(
            call_code(
                help_d=ss["call_data"],
                key=keyval,
            ),
            language="python",
        )

        st.subheader("API Sonucu" if ss["lang"] == "tr" else "API Result")

        get_result = st.button(
            "API Sonucunu Getir" if ss["lang"] == "tr" else "Get API Result"
        )

        if get_result:

            try:
                bod_params = call_code(
                    help_d=ss["call_data"],
                    key=d2[ss["eptr2_call"]]["key"],
                    just_body_params=True,
                )

                res = ss["eptr"].call(d2[ss["eptr2_call"]]["key"], **bod_params)
                res_df = get_postprocess_function(keyval)(res=res, key=keyval)
                st.dataframe(res_df)
            except Exception as e:
                st.warning("Bu API için örnek çağırma yapılamadı.")


# st.divider()
# st.subheader("Debug")
# st.json(ss)



from eptr2.mapping.path import get_path_map
from eptr2.mapping.parameters import get_required_parameters


def check_path_parameter_parity():

    path_keys = get_path_map(just_call_keys=True)
    param_keys = get_required_parameters(
        key="", return_mapping=True, mapping_only_keys=True
    )

    missing_param_keys = [x for x in path_keys if x not in param_keys]
    missing_path_keys = [x for x in param_keys if x not in path_keys]
    if len(missing_param_keys) > 0:
        print("Missing parameters for paths:")
        print(missing_param_keys)
    if len(missing_path_keys) > 0:
        print("Missing paths for parameters:")
        print(missing_path_keys)

    if len(missing_param_keys) == 0 and len(missing_path_keys) == 0:
        print("Path and parameter parity check passed.")
        return True

    return missing_param_keys, missing_path_keys


if __name__ == "__main__":
    check_path_parameter_parity()



from eptr2 import EPTR2
import pandas as pd


def get_hourly_consumption_and_forecast_data(
    eptr: EPTR2, start_date: str, end_date: str, verbose: bool = False
):
    """
    This composite function gets load plan, UECM (settlement consumption), real time and consumption data. If end date is after the last settlement data, UECM is filled with real time consumption under consumption column.
    """

    if verbose:
        print("Loading load plan...")

    lp_df = eptr.call("load-plan", start_date=start_date, end_date=end_date)

    df = lp_df[["date", "lep"]].rename(columns={"lep": "load_plan", "date": "dt"})

    if verbose:
        print("Loading UECM...")

    uecm_df = eptr.call("uecm", start_date=start_date, end_date=end_date)

    df = df.merge(
        uecm_df[["period", "swv"]].rename(columns={"period": "dt", "swv": "uecm"}),
        on="dt",
        how="outer",
    )

    if verbose:
        print("Loading real time consumption...")

    rt_cons = eptr.call("rt-cons", start_date=start_date, end_date=end_date)

    df = df.merge(
        rt_cons[["date", "consumption"]].rename(
            columns={"date": "dt", "consumption": "rt_cons"}
        ),
        on="dt",
        how="outer",
    )

    df["consumption"] = df.apply(
        lambda x: x["rt_cons"] if pd.isnull(x["uecm"]) else x["uecm"], axis=1
    )

    return df



from datetime import datetime, timedelta


def format_date_epias_hour(date: str | datetime, transform: str | None = None):
    """
    Desired format "2023-01-01T00:00:00+03:00"
    """
    if isinstance(date, str):
        date = datetime.fromisoformat(date)

    date = date.strftime("%Y-%m-%dT00:00:00+03:00")

    return date


def preprocess_parameter(key, value):
    if key in [
        "start_date",
        "end_date",
        "date",
        "period",
        "se_date",
        "period_start_date",
        "period_end_date",
        "version_start_date",
        "version_end_date",
    ]:
        value = format_date_epias_hour(value)

    elif key in ["region"]:
        value = "TR1" if value is None else value

    elif key in ["region_id"]:
        value = "1" if value is None else value

    elif key in ["price_type"]:
        value = "SMP" if value is None else value
        if value not in ["MCP", "SMP"]:
            raise Exception("Price type must be either MCP or SMP")

    elif key in ["point_type"]:
        if value not in ["INPUT", "OUTPUT"]:
            raise Exception("Point type must be either INPUT or OUTPUT")

    elif key in ["intl_direction"]:
        # value = "TRGR" if value is None else value
        val_list = ["TRGR", "GRTR", "TRBG", "BGTR"]
        if value not in val_list:
            raise Exception(
                "International direction (intl_direction) type should be one of "
                + ", ".join(val_list)
                + "."
            )

    elif key in ["order_type"]:
        value = "BOTH_REGULATIONS" if value is None else value
        if value not in ["UP_REGULATION", "DOWN_REGULATION", "BOTH_REGULATIONS"]:
            raise Exception(
                "Order type must be either UP_REGULATION, DOWN_REGULATION or BOTH_REGULATIONS"
            )

    elif key in ["load_type"]:
        value = "Baz" if value is None else value
        if value not in ["Baz", "Puant", "Puant Dışı"]:
            raise Exception("Load type must be either Baz, Puant or Puant Dışı")

    return value



import pandas as pd
import json


def postprocess_direct_dict(res, key: str | None = None):
    ## for date-init
    subkey_map = {
        "dam-list": "damList",
    }

    subkey = subkey_map.get(key, None)
    d = res
    try:
        if subkey is not None:
            return d[subkey]
    except Exception as e:
        print(e)

    return d


def postprocess_direct_dict_to_df(res, **kwargs):
    ## for date-init
    return pd.DataFrame(res)


def postprocess_mcp_status(res, **kwargs):
    return res["body"]["content"]["completed"]


def postprocess_items_to_df(res, **kwargs):
    df = pd.DataFrame(res["items"])
    return df


## Adhoc non-standard response postprocess
def postprocess_ren_capacity_dict_to_df(res, **kwargs):
    df = pd.DataFrame(res["installedCapacities"])
    return df



def get_kupst_tolerance(source: str):
    tol_source_map = {"wind": 0.21, "solar": 0.12}
    return tol_source_map.get(source, 0.05)


def calculate_unit_imbalance_price(
    mcp: float, smp: float, penalty_margin: float = 0.03
):
    d = {
        "pos": (1 - penalty_margin) * min(mcp, smp),
        "neg": (1 + penalty_margin) * max(mcp, smp),
    }
    return d


def calculate_unit_imbalance_cost(mcp: float, smp: float, penalty_margin: float = 0.03):
    d = calculate_unit_imbalance_price(mcp=mcp, smp=smp, penalty_margin=penalty_margin)

    d["pos"] = mcp - d["pos"]
    d["neg"] = d["neg"] - mcp

    return d


def calculate_unit_kupst_cost(mcp: float, smp: float, kupst_multiplier: float = 0.03):
    return max(mcp, smp) * kupst_multiplier


def calculate_imbalance_cost(
    actual: float,
    forecast: float,
    mcp: float,
    smp: float,
    is_producer: bool,
    imbalance_discount: float = 0.0,
):
    if imbalance_discount > 1 or imbalance_discount < 0:
        raise ValueError("Imbalance dampening rate must be between 0 and 1")

    unit_cost_d = calculate_unit_imbalance_cost(mcp=mcp, smp=smp)

    diff = forecast - actual
    if is_producer:
        cost = abs(diff) * unit_cost_d["neg" if diff > 0 else "pos"]
    else:
        cost = abs(diff) * unit_cost_d["pos" if diff > 0 else "neg"]

    return cost * (1 - imbalance_discount)


def calculate_kupst_cost(
    actual: float,
    forecast: float,
    mcp: float,
    smp: float,
    tol: float | None = None,
    source: str | None = None,
    kupst_multiplier: float = 0.03,
):
    if tol is None:
        if source is None:
            raise Exception("Either tol(erance) or source parameter must be provided")
        tol = get_kupst_tolerance(source)

    tol_val = forecast * tol
    kupst_cost = max(0, abs(actual - forecast) - tol_val) * calculate_unit_kupst_cost(
        mcp, smp, kupst_multiplier
    )
    return kupst_cost


def calculate_diff_cost(
    actual: float,
    forecast: float,
    mcp: float,
    smp: float,
    tol: float | None = None,
    kupst_multiplier: float = 0.03,
    is_producer: bool = True,
    source: str | None = None,
    imbalance_discount: float = 0.0,
):
    imbalance = calculate_imbalance_cost(
        actual=actual,
        forecast=forecast,
        mcp=mcp,
        smp=smp,
        is_producer=is_producer,
        imbalance_discount=imbalance_discount,
    )

    kupst = calculate_kupst_cost(
        actual=actual,
        forecast=forecast,
        mcp=mcp,
        smp=smp,
        tol=tol,
        source=source,
        kupst_multiplier=kupst_multiplier,
    )

    total = imbalance + kupst

    return {"imbalance": imbalance, "kupst": kupst, "total": total}


def calculate_imbalance_prices_list(mcp: list, smp: list):
    """
    Calculates imbalance prices for positive and negative imbalances
    """
    d = {
        "pos": [min(x, y) * 0.97 for x, y in zip(mcp, smp)],
        "neg": [1.03 * max(x, y) for x, y in zip(mcp, smp)],
    }

    return d


def calculate_imbalance_cost_values_list(mcp: list, smp: list):
    """
    Calculates imbalance costs relative to day-ahead prices (MCP) for positive and negative imbalances
    """
    d = calculate_imbalance_prices_list(mcp, smp)
    d["pos"] = [x - y for x, y in zip(mcp, d["pos"])]
    d["neg"] = [y - x for x, y in zip(mcp, d["neg"])]

    return d


def calculate_kupst_cost_list(
    actual_values: list,
    forecast_values: list,
    mcp: list,
    smp: list,
    tol: float | None = None,
    source: str | None = None,
    kupst_multiplier: float = 0.03,
    min_kupst: float = 750.0,
):
    """
    Calculates production plan difference (KUPST) costs.
    """

    if tol is None:
        if source is None:
            raise Exception("Either tol(erance) or source parameter must be provided")
        tol = get_kupst_tolerance(source)

    kupst_cost_list = []

    for x, y, m, s in zip(forecast_values, actual_values, mcp, smp):
        cost = max(abs(x - y) - tol * x, 0) * max(m, s) * kupst_multiplier
        cost = max(cost, min_kupst)

        kupst_cost_list.append(cost)

    return kupst_cost_list


def calculate_imbalance_cost_list(
    actual_values: list,
    forecast_values: list,
    cost_d: dict,
    is_producer: bool,
    imbalance_discount: float = 0.0,
):
    """
    Calculates imbalance costs for a given forecast and actual values.
    Imbalance dampening reduces imbalance costs by the given rate (<1). It is used to emulate balance responsible party (DSG) effect.
    """

    if imbalance_discount > 1 or imbalance_discount < 0:
        raise ValueError("Imbalance dampening rate must be between 0 and 1")

    if len(actual_values) != len(forecast_values):
        raise ValueError("Actual and forecast values must have the same length")

    imbalance_cost_list = []

    for x, y, n, p in zip(forecast_values, actual_values, cost_d["neg"], cost_d["pos"]):
        if x < y:
            cost = (y - x) * p if is_producer else n
        else:
            cost = (x - y) * n if is_producer else p

        imbalance_cost_list.append(cost * (1 - imbalance_discount))

    return imbalance_cost_list


def calculate_diff_costs_list(
    actual_values: list,
    forecast_values: list,
    mcp: list,
    smp: list,
    is_producer: bool,
    tol: float | None = None,
    source: str | None = None,
    kupst_multiplier: float = 0.03,
    imbalance_discount: float = 0,
    min_kupst: float = 0.0,
):
    """
    Wrapper function to calculate all costs related to the difference between actual and forecasted values..
    """
    res_d = {"actual": actual_values, "forecast": forecast_values}

    cost_d = calculate_imbalance_cost_values_list(mcp, smp)
    res_d["diff"] = [x - y for x, y in zip(forecast_values, actual_values)]

    if is_producer:
        res_d["imbalance_direction"] = [
            "pos" if x < 0 else "neg" for x in res_d["diff"]
        ]
    else:
        res_d["imbalance_direction"] = [
            "pos" if x > 0 else "neg" for x in res_d["diff"]
        ]

    res_d["imbalance_cost"] = calculate_imbalance_cost_list(
        actual_values=actual_values,
        forecast_values=forecast_values,
        cost_d=cost_d,
        is_producer=is_producer,
        imbalance_discount=imbalance_discount,
    )

    if is_producer:
        res_d["kupst_cost"] = calculate_kupst_cost_list(
            actual_values=actual_values,
            forecast_values=forecast_values,
            mcp=mcp,
            smp=smp,
            tol=tol,
            source=source,
            kupst_multiplier=kupst_multiplier,
            min_kupst=min_kupst,
        )

    return res_d



from eptr2 import EPTR2
from eptr2.util.costs import calculate_unit_kupst_cost
import pandas as pd


def get_hourly_price_and_cost_data(
    eptr: EPTR2,
    start_date: str,
    end_date: str,
    include_wap: bool = True,
    add_kupst_cost: bool = True,
    verbose: bool = False,
):
    """
    This composite function gets price and imbalance (kupst included) cost data.
    Imbalance cost data consists of MCP (PTF), SMP (SMF), system direction (Enerji Açığı - up regulated, Energy Fazlası - down regulated, Dengede - Balanced), positive imbalance cost and negative imbalance cost.

    Resulting columns are:
    - date: Datetime in ISO format and +03:00 timezone
    - mcp: Market Clearing Price (MCP/PTF) in TL (Turkish Lira)
    - smp: System Marginal Price (SMP/SMF) in TL (Turkish Lira)
    - wap: Weighted Average Price (WAP) in TL (Turkish Lira)
    - system_direction: System direction (Enerji Açığı - up regulated, Energy Fazlası - down regulated, Dengede - Balanced)
    - sd_sign: System direction sign (1 for Enerji Fazlası, -1 for Enerji Açığı, 0 for Dengede)
    - pos_imb_price: Positive imbalance price in TL (Turkish Lira)
    - neg_imb_price: Negative imbalance price in TL (Turkish Lira) (negative value)
    - pos_imb_cost: Positive imbalance cost in TL (Turkish Lira) (ptf - pos_imb_price)
    - neg_imb_cost: Negative imbalance cost in TL (Turkish Lira) (neg_imb_price - ptf)
    """

    if verbose:
        print("Getting MCP, SMP and imbalance price data...")

    price_df = eptr.call("mcp-smp-imb", start_date=start_date, end_date=end_date)

    price_df.drop(columns=["time"], inplace=True)

    price_df.rename(
        columns={
            "ptf": "mcp",
            "smf": "smp",
            "positiveImbalance": "pos_imb_price",
            "negativeImbalance": "neg_imb_price",
            "systemStatus": "system_direction",
        },
        inplace=True,
    )

    price_df["sd_sign"] = price_df["system_direction"].apply(
        lambda x: -1 if x == "Enerji Açığı" else 1 if x == "Enerji Fazlası" else 0
    )

    price_df["pos_imb_cost"] = price_df["mcp"] - price_df["pos_imb_price"]
    price_df["neg_imb_cost"] = price_df["neg_imb_price"] - price_df["mcp"]

    if include_wap:
        if verbose:
            print("Getting WAP data...")
        wap_df = eptr.call("wap", start_date=start_date, end_date=end_date)
        wap_df.drop(columns=["hour"], inplace=True)

        price_df = price_df.merge(wap_df, on="date", how="outer")

    if add_kupst_cost:
        if verbose:
            print("Calculating unit KUPST cost...")

        price_df["kupst_cost"] = price_df.apply(
            lambda x: calculate_unit_kupst_cost(mcp=x["mcp"], smp=x["smp"]), axis=1
        )

    ### Column ordering
    columns_order = [
        "date",
        "pos_imb_vol",
        "neg_imb_vol",
        "pos_imb_mwh",
        "neg_imb_mwh",
        "mcp",
        "wap",
        "smp",
        "pos_imb_price",
        "neg_imb_price",
        "system_direction",
        "sd_sign",
        "pos_imb_cost",
        "neg_imb_cost",
        "kupst_cost",
    ]

    columns_order = [col for col in columns_order if col in price_df.columns]

    price_df = price_df[columns_order]

    return price_df


def get_hourly_imbalance_data(
    eptr: EPTR2,
    start_date: str,
    end_date: str,
    include_price_and_cost_data: bool = True,
    include_wap: bool = True,
    add_kupst_cost: bool = True,
    verbose: bool = False,
):
    """
    This composite function gets imbalance volume, imbalance quantity and imbalance cost data.
    Imbalance cost data consists of MCP (PTF), SMP (SMF), system direction (Enerji Açığı - up regulated, Energy Fazlası - down regulated, Dengede - Balanced), positive imbalance cost and negative imbalance cost.

    Resulting columns are:
    - date: Datetime in ISO format and +03:00 timezone
    - pos_imb_vol: Positive imbalance volume in TL (Turkish Lira)
    - neg_imb_vol: Negative imbalance volume in TL (Turkish Lira) (negative value)
    - pos_imb_mwh: Positive imbalance quantity in MWh
    - neg_imb_mwh: Negative imbalance quantity in MWh (negative value)
    - mcp: Market Clearing Price (MCP/PTF) in TL (Turkish Lira)
    - smp: System Marginal Price (SMP/SMF) in TL (Turkish Lira)
    - system_direction: System direction (Enerji Açığı - up regulated, Energy Fazlası - down regulated, Dengede - Balanced)
    - sd_sign: System direction sign (1 for Enerji Fazlası, -1 for Enerji Açığı, 0 for Dengede)
    - pos_imb_price: Positive imbalance price in TL (Turkish Lira)
    - neg_imb_price: Negative imbalance price in TL (Turkish Lira) (negative value)
    - pos_imb_cost: Positive imbalance cost in TL (Turkish Lira) (ptf - pos_imb_price)
    - neg_imb_cost: Negative imbalance cost in TL (Turkish Lira) (neg_imb_price - ptf)
    """

    imb_vol_df = eptr.call(
        "imb-vol", start_date=start_date, end_date=end_date, verbose=verbose
    )
    imb_vol_df.drop(columns=["hour"], inplace=True)
    imb_vol_df.rename(
        columns={
            "positiveImbalance": "pos_imb_vol",
            "negativeImbalance": "neg_imb_vol",
        },
        inplace=True,
    )

    imb_vol_df["neg_imb_vol"] = -imb_vol_df["neg_imb_vol"]

    imb_qty_df = eptr.call(
        "imb-qty", start_date=start_date, end_date=end_date, verbose=verbose
    )
    imb_qty_df.drop(columns=["hour"], inplace=True)

    imb_qty_df.rename(
        columns={
            "positiveImbalance": "pos_imb_mwh",
            "negativeImbalance": "neg_imb_mwh",
        },
        inplace=True,
    )

    imb_qty_df["neg_imb_mwh"] = -imb_qty_df["neg_imb_mwh"]

    merged_df = imb_vol_df.merge(imb_qty_df, on="date", how="outer")

    if not include_price_and_cost_data:
        return merged_df

    price_df = get_hourly_price_and_cost_data(
        eptr=eptr,
        start_date=start_date,
        end_date=end_date,
        include_wap=include_wap,
        add_kupst_cost=add_kupst_cost,
        verbose=verbose,
    )

    merged_df = merged_df.merge(price_df, on="date", how="outer")

    return merged_df
